@@ -141,7 +141,7 @@ public:
                                         boost::asio::ip::tcp::endpoint &) >
         tcp_schedule_func_type;
 
-        typedef l7vs::atomic<unsigned long long> AUUL;
+        typedef l7vs::atomic<unsigned long long> AULL;
         //typedef l7vs::atomic<unsigned int> AUI;
 
 
@@ -178,7 +178,7 @@ public:
         const static int  SCHEDULER_PRIORITY       = 20;
 
         //! Default upstream QoS alert on threshold
-	const static int  UPQOS_ALERT_ON_SIZE_DEFAULT       = 85;
+        const static int  UPQOS_ALERT_ON_SIZE_DEFAULT       = 85;
         //! Default upstream QoS alert off threshold
         const static int  UPQOS_ALERT_OFF_SIZE_DEFAULT      = 50;
         //! Default downstream QoS alert on threshold
@@ -188,8 +188,9 @@ public:
         //! Default session pool alert on threshold
         const static int  SESSIONPOOL_ALERT_ON_SIZE_DEFAULT = 5;
         //! Default session pool alert off threshold
-        const static int  SESSIONPOOL_ALERT_OFF_SIZE_DEFAULT= 8;
+        const static int  SESSIONPOOL_ALERT_OFF_SIZE_DEFAULT = 8;
 
+        static const int IO_SERVICE_THREADS_NUM = 1;
 protected:
 
         struct    parameter_data {
@@ -204,15 +205,15 @@ protected:
                 unsigned long long session_pool_alert_on;
                 unsigned long long session_pool_alert_off;
                 parameter_data() :  nic_realserver_side(RS_SIDE_NIC_NAME_DEFAULT),
-                                session_pool_size(SESSION_POOL_NUM_DEFAULT),
-                                bps_interval(BPS_INTERVAL_DEFAULT),
-                                rep_interval(REP_INTERVAL_DEFAULT),
-				qos_up_alert_on(UPQOS_ALERT_ON_SIZE_DEFAULT),
-                                qos_up_alert_off(UPQOS_ALERT_OFF_SIZE_DEFAULT),
-                                qos_down_alert_on(DOWNQOS_ALERT_ON_SIZE_DEFAULT),
-                                qos_down_alert_off(DOWNQOS_ALERT_OFF_SIZE_DEFAULT),
-                                session_pool_alert_on(SESSIONPOOL_ALERT_ON_SIZE_DEFAULT),
-                                session_pool_alert_off(SESSIONPOOL_ALERT_OFF_SIZE_DEFAULT) {}
+                        session_pool_size(SESSION_POOL_NUM_DEFAULT),
+                        bps_interval(BPS_INTERVAL_DEFAULT),
+                        rep_interval(REP_INTERVAL_DEFAULT),
+                        qos_up_alert_on(UPQOS_ALERT_ON_SIZE_DEFAULT),
+                        qos_up_alert_off(UPQOS_ALERT_OFF_SIZE_DEFAULT),
+                        qos_down_alert_on(DOWNQOS_ALERT_ON_SIZE_DEFAULT),
+                        qos_down_alert_off(DOWNQOS_ALERT_OFF_SIZE_DEFAULT),
+                        session_pool_alert_on(SESSIONPOOL_ALERT_ON_SIZE_DEFAULT),
+                        session_pool_alert_off(SESSIONPOOL_ALERT_OFF_SIZE_DEFAULT) {}
         };
 
         boost::thread::id       this_id;
@@ -221,8 +222,9 @@ protected:
         const replication      &rep;                  //! replication reference
 
         Logger                  log;
-
-        boost::asio::io_service dispatcher;           //! dispatcher service
+        boost::shared_ptr< boost::asio::io_service >
+        dispatcher;           //! dispatcher service
+        boost::shared_ptr< boost::asio::io_service::work >      work;
         deadline_timer_ptr_type calc_bps_timer;       //! timer object
         deadline_timer_ptr_type replication_timer;    //! timer object
         deadline_timer_ptr_type protomod_rep_timer;   //! timer object
@@ -236,36 +238,40 @@ protected:
         schedule_module_base   *schedmod;      //! schedule module smart pointer
 
         std::list<realserver>   rs_list;              //! realserver list
-        AUUL                    rs_list_ref_count;    //! reference count of
+        AULL                    rs_list_ref_count;    //! reference count of
         //! realserver list
         wr_mutex                rs_list_ref_count_inc_mutex; //! mutex for
         //! increase reference count
 
-        AUUL recvsize_up;           //! upstream total receive data size
-        AUUL current_up_recvsize;   //! current upstream receive data size
+        AULL recvsize_up;           //! upstream total receive data size
+        AULL current_up_recvsize;   //! current upstream receive data size
         //!  for calculate upstream throughput
-        AUUL sendsize_up;           //! upstream total send data size
-        AUUL recvsize_down;         //! downstream total receive data size
-        AUUL current_down_recvsize; //! current downstream receive data size for
+        AULL sendsize_up;           //! upstream total send data size
+        AULL recvsize_down;         //! downstream total receive data size
+        AULL current_down_recvsize; //! current downstream receive data size for
         //! calculate upstream throughput
-        AUUL sendsize_down;         //! downstream total send data size
+        AULL sendsize_down;         //! downstream total send data size
 
-        AUUL throughput_up;         //! upstream throughput value
-        AUUL throughput_down;       //! downstream throughput value
+        AULL throughput_up;         //! upstream throughput value
+        AULL throughput_down;       //! downstream throughput value
 
-        AUUL wait_count_up;         //! upstream recv wait count
-        AUUL wait_count_down;       //! downstream recv wait count
+        AULL wait_count_up;         //! upstream recv wait count
+        AULL wait_count_down;       //! downstream recv wait count
 
-        AUUL interrupt_running_flag;    //! interrupt routine is running
-        AUUL virtualservice_stop_flag;  //! virtualservice is now shutting down
+        AULL interrupt_running_flag;    //! interrupt routine is running
+        AULL virtualservice_stop_flag;  //! virtualservice is now shutting down
 
         // protocol module option string
         std::string             protocol_module_for_indication_options;
 
-	bool upqos_alert_flag;          //! upstream QoS alert flag
+        bool upqos_alert_flag;          //! upstream QoS alert flag
         bool downqos_alert_flag;        //! downstream QoS alert flag
         bool sessionpool_alert_flag;    //! sessionpool alert flag
 
+        bool adm_cmd_wait_flag;         //! wait for l7vsadm done
+        boost::mutex adm_cmd_wait_flag_mutex;
+        boost::condition adm_cmd_wait_flag_cond;
+ 
         void         load_parameter(l7vs::error_code &);
 
         virtual void handle_replication_interrupt(
@@ -320,8 +326,8 @@ public:
                                      const replication &,
                                      const virtualservice_element &);
         virtual ~virtualservice_base() {
-                dispatcher.reset();
-                dispatcher.stop();
+                dispatcher->reset();
+                dispatcher->stop();
         };
 
         virtual void initialize(error_code &) = 0;
@@ -330,8 +336,7 @@ public:
         virtual bool operator==(const virtualservice_base &) = 0;
         virtual bool operator!=(const virtualservice_base &) = 0;
 
-        void rs_list_lock() {
-                rd_scoped_lock lock(rs_list_ref_count_inc_mutex);
+        void rs_list_lock() {   // rs_list_ref_cont is atomic numeric. this numeric not use lock.
                 rs_list_ref_count++;
         }
         void rs_list_unlock() {
@@ -422,6 +427,8 @@ public:
         }
 
         virtual void clear_inact();
+
+        virtual bool is_realserver_transparent(const boost::asio::ip::tcp::endpoint &);
 };
 
 //!
@@ -437,14 +444,15 @@ public:
         typedef std::map< std::string, std::string >
         accesslog_argument_map_type;
 protected:
-        boost::asio::ip::tcp::acceptor
-        acceptor_;
+        boost::shared_ptr<      boost::asio::ip::tcp::acceptor > acceptor_;
 
         session_queue_type          pool_sessions;
-        session_map_type            waiting_sessions;
+        tcp_session                            *waiting_session;
         session_map_type            active_sessions;
         l7vs::atomic<unsigned long long>
         active_count;
+        session_thread_control      *waiting_stc;
+        //! session_thread_control waiting accept
 
         bool                        defer_accept_opt;
         //! is set option TCP_DEFER_ACCEPT
@@ -457,7 +465,8 @@ protected:
         bool                        ssl_virtualservice_mode_flag;
         std::string                 ssl_file_name;
         // SSL context
-        boost::asio::ssl::context   sslcontext;
+        boost::shared_ptr<boost::asio::ssl::context>
+        sslcontext;
         // SSL context parameter
         std::string                 ca_dir;
         std::string                 ca_file;
@@ -537,10 +546,6 @@ public:
                 return pool_sessions.size();
         }
 
-        int get_waiting_sessions_count() {
-                return waiting_sessions.size();
-        }
-
         int get_active_sessions_count() {
                 return active_sessions.size();
         }
@@ -641,10 +646,6 @@ public:
                 return vs->get_pool_sessions_count();
         }
 
-        int get_waiting_sessions_count() {
-                return vs->get_waiting_sessions_count();
-        }
-
         int get_active_sessions_count() {
                 return vs->get_active_sessions_count();
         }
@@ -652,6 +653,10 @@ public:
         void clear_inact() {
                 return vs->clear_inact();
         }
+
+        bool is_realserver_transparent(const boost::asio::ip::tcp::endpoint &endpoint) {
+                return vs->is_realserver_transparent(endpoint);
+        }
 };
 
 } //namespace l7vs
